%
%
%  Histogram Equalize Layers for Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory.
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%        <Item name="Histogram Equalize Layers" icon="Matlab" tooltip="Equalizes the histogram of voxel intensity layer by layer.">
%          <Command>MatlabXT::XTHistogramEqualizeLayers(%i)</Command>
%        </Item>
%      </Menu>
%    </CustomTools>
%  
%
%  Description:
%
%   Equalizes the histogram of voxel intensity layer by layer.
%
%

function XTHistogramEqualizeLayers(aImarisApplicationID, aExcludeZero)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
    javaaddpath ImarisLib.jar
    vImarisLib = ImarisLib;
    if ischar(aImarisApplicationID)
        aImarisApplicationID = round(str2double(aImarisApplicationID));
    end
    vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
    vImarisApplication = aImarisApplicationID;
end

vThreshold = 0;
if nargin == 1 
   vAnswer = inputdlg({'Exclude Voxels less than'}, 'Histogram Equalize Layers', 1, {'1'});
   if ~isempty(vAnswer)
       vThreshold = str2double(vAnswer(1)); 
   else 
       return;
   end
elseif nargin == 2
    if aExcludeZero
        vThreshold = 1;
    end
end

try
  vImarisApplication.DataSetPushUndo('Histogram Equalize Layers');
catch er
  % nothing to do
  er.message;
end

try
  % get the data set 
  vDataSet = vImarisApplication.GetDataSet.Clone;
  vNumberOfTimePoints = vDataSet.GetSizeT;
catch er
  msgbox(['Invalid dataset\n\n', er.message])
  return
end

vNumberOfChannels = vDataSet.GetSizeC;
vNumberOfChannelsSelected = 0;
for vChannelIndex = 1:vNumberOfChannels
    if vImarisApplication.GetChannelVisibility(vChannelIndex-1)
        vNumberOfChannelsSelected = vNumberOfChannelsSelected + 1;
    end
end
vNumberOfSlices = vDataSet.GetSizeZ;

vImageIsInteger = strcmp(vImarisApplication.GetDataSet.GetType,'eTypeUInt8') || ...
        strcmp(vImarisApplication.GetDataSet.GetType,'eTypeInt16');

vSize = [vDataSet.GetSizeX,vDataSet.GetSizeY,vDataSet.GetSizeZ];
      
vTotalCount = vNumberOfSlices*vNumberOfChannelsSelected*vNumberOfTimePoints;
vProgressDisplay = waitbar(0,'Histogram Equalize Layers');
vProgressCount = 0;

for vChannelIndex = 1:vNumberOfChannels
    if vImarisApplication.GetChannelVisibility(vChannelIndex-1) == false
        continue;
    end
    
    for vTimeIndex = 1:vNumberOfTimePoints
        if strcmp(vDataSet.GetType,'eTypeUInt8')
            vImage = typecast(vDataSet.GetDataVolumeAs1DArrayBytes( ...
                vChannelIndex-1, vTimeIndex-1), 'uint8');
        elseif strcmp(vDataSet.GetType, 'eTypeUInt16')
            vImage = typecast(vDataSet.GetDataVolumeAs1DArrayShorts( ...
                vChannelIndex-1, vTimeIndex-1), 'uint16');
        elseif strcmp(vDataSet.GetType, 'eTypeFloat')
            vImage = vDataSet.GetDataVolumeAs1DArrayFloats( ...
                vChannelIndex-1, vTimeIndex-1);
        end
        vImage = reshape(double(vImage), vSize);
        
        vDataVector = vImage(vImage>=vThreshold);
        vCount = numel(vDataVector);
        if vCount > 0
            % histogram
            vMin = min(vDataVector);
            vMax = max(vDataVector);
            if vImageIsInteger
                vNBins = vMax-vMin;
            else
                vNBins = 2^16;
            end
            vHistogram = hist((vDataVector-vMin)/(vMax-vMin),vNBins);
            vMean = mean(vDataVector);
            for vSlice = 1:size(vImage,3)
                vImageSlice = vImage(:,:,vSlice);
                vDataVectorSlice = vImageSlice(vImageSlice>=vThreshold);
                vCountSlice = numel(vDataVectorSlice);
                if vCountSlice > 0
                    vVarianceSlice = var(vDataVectorSlice);
                    if vVarianceSlice == 0;
                        vImageSlice(vImageSlice>=vThreshold) = zeros(size(vDataVectorSlice))+vMean;
                    else
                        vImageSlice(vImageSlice>=vThreshold) = vMin + (vMax-vMin) * histeq((vDataVectorSlice-vMin)/(vMax-vMin), vHistogram);
                    end
                    vImage(:,:,vSlice) = vImageSlice;
                end
                vProgressCount = vProgressCount + 1;
                waitbar(vProgressCount/vTotalCount)
            end
        else
            vProgressCount = vProgressCount + vNumberOfSlices;
            waitbar(vProgressCount/vTotalCount)
        end
        
        if strcmp(vImarisApplication.GetDataSet.GetType,'eTypeUInt8') 
            vDataSet.SetDataVolumeBytes(uint8(vImage), vChannelIndex-1, vTimeIndex-1);
        elseif strcmp(vImarisApplication.GetDataSet.GetType,'eTypeInt16') 
            vDataSet.SetDataVolumeShorts(uint16(vImage), vChannelIndex-1, vTimeIndex-1);
        else strcmp(vImarisApplication.GetDataSet.GetType,'eTypeFloat')    
            vDataSet.SetDataVolumeFloats(single(vImage), vChannelIndex-1, vTimeIndex-1);
        end
    end
end
close(vProgressDisplay);

vImarisApplication.SetDataSet(vDataSet);
